from abc import ABC, abstractmethod

class BolognaStrategy(ABC):
    """
    Abstract class for Bologna strategies
    """

    def __init__(self):
        """
        Init of class
        """
        self.TLS_DETECTORS = {
            '209': self.DETECTORS_209,
            '210': self.DETECTORS_210,
            '219': self.DETECTORS_219,
            '220': self.DETECTORS_220,
            '221': self.DETECTORS_221,
            '235': self.DETECTORS_235,
            '273': self.DETECTORS_273
        }

    TL_IDS = ['209', '210', '219', '220', '221', '235', '273']

    DETECTORS_210 = {
        0: {
            'boolean': ['34_3', '34_4', '34_5', '27_1', '27_2', '27_3', '27_7', '27_8'],
            'saturation': ['34_7', '34_8', '34_9', '27_16', '27_17'], #'27_13', '27_14', '27_15',,
            'numerical': ['34_13', '34_14', '34_15', '27_20', '27_21', '27_22', '27_28', '27_29', '27_31', '27_32'],
            'exit': ['1_21', '1_22', '1_23']
        },
        2: {
            'boolean': ['34_1', '34_2', '27_9', '27_3'],
            'saturation': ['27_18'],
            'numerical': ['34_10', '34_11', '34_12', '27_22', '27_30', '27_30'],
            'exit': ['1_21', '1_22', '1_23']
        },
        4: {
            'boolean': ['34_1', '34_2', '27_4', '27_5', '27_6', '27_10', '27_11', '27_12'],
            'saturation': [],
            'numerical': ['34_10', '34_11', '34_12', '27_34', '27_35', '27_36', '27_37', '27_23', '27_24', '27_25', '27_26', '27_27'],
            'exit': ['1_21', '1_22', '1_23']
        },
    }

    DETECTORS_221 = {
        0: {
            'boolean': ['1_1', '1_2', '1_3', '1_4', '1_5', '1_6'],
            'saturation': ['1_12', '1_13', '1_14', '1_15', '1_16', '1_17'],
            'numerical': ['1_21', '1_22', '1_23', '1_24', '1_25', '1_26'],
            'exit': ['78_17', '78_18', '78_19', '27_31', '27_32', '27_33']
        },
        3: {
            'boolean': ['1_7', '1_8', '1_9', '1_10', '1_11'],
            'saturation': ['1_18', '1_19', '1_20'],
            'numerical': ['1_27', '1_28', '1_29', '1_30', '1_31', '1_32', '1_33', '1_34', '1_35', '1_36'],
            'exit': ['78_17', '78_18', '27_31', '27_32', '27_33']
        }
    }

    DETECTORS_235 = {
        0: {
            'boolean': ['78_4', '78_5', '78_6'],
            'saturation': ['78_11', '78_12', '78_13'],
            'numerical': ['78_17', '78_18', '78_19'],
            'exit': []
        },
        4: {
            'boolean': ['78_7', '78_8', '78_9'],
            'saturation': [],
            'numerical': [],
            'exit': []
        },
        6: {
            'boolean': ['78_1', '78_2', '78_3'],
            'saturation': [],
            'numerical': ['78_14', '78_15', '78_16'],
            'exit': ['1_24', '1_25', '1_26']
        },
        8: {
            'boolean': ['78_10'],
            'saturation': [],
            'numerical': [],
            'exit': []
        },
    }

    DETECTORS_220 = {
        0: {
            'boolean': ['15_3', '15_4', '15_5', '52_3'],
            'saturation': ['15_12', '15_10', '15_11', '52_6'],
            'numerical': ['15_23', '15_18', '15_19', '15_20', '15_21', '15_22', '52_9'],
            'exit': ['9_22', '9_23', '9_24', '9_25', '9_26']
        },
        2: {
            'boolean': ['15_1', '15_2', '52_1', '52_2', '15_24', '15_25', '15_26'],
            'saturation': ['15_6', '15_7', '15_8', '15_9', '52_4', '52_5'],
            'numerical': ['15_13', '15_14', '15_15', '15_16', '15_17', '15_18', '52_7', '52_8'],
            'exit': ['52_7', '52_8']
        }
    }

    DETECTORS_219 = {
        0: {
            'boolean': ['9_7', '9_8', '9_9', '9_1', '9_2', '9_3'],
            'saturation': ['9_16', '9_21'],
            'numerical': ['9_27', '9_30', '9_18', '9_19', '9_20', '9_21'],
            'exit': ['15_16', '15_13', '15_14']
        },
        # 2: {
        #     'boolean': ['9_7', '9_8', '9_9'],
        #     'saturation': ['9_16', '9_17'],
        #     'numerical': ['9_27', '9_28', '9_29', '9_30', '9_31'],
        #     'exit': ['15_16', '15_13', '15_14']
        # },
        2: {
            'boolean': ['9_4', '9_5', '9_10', '9_11'],
            'saturation': ['9_14'],
            'numerical': ['9_22', '9_23', '9_25', '9_32', '9_33'],
            'exit': ['15_13', '15_14', '15_15', '15_16', '15_17']
        },
        4: {
            'boolean': ['9_6', '9_12'],
            'saturation': ['9_15'],
            'numerical': ['9_24', '9_26', '9_34'],
            'exit': []
        }
    }

    DETECTORS_273 = {
        0: {
            'boolean': ['12_4', '12_1', '12_2'],
            'saturation': ['12_9', '12_6'],
            'numerical': ['12_10', '12_11', '12_15', '12_13'],
            'exit': []
        },
        2: {
            'boolean': ['12_2', '12_5'],
            'saturation': ['12_9', '12_7'],
            'numerical': ['12_14', '12_11', '12_15'],
            'exit': []
        },
        4: {
            'boolean': ['12_3'],
            'saturation': ['12_8'],
            'numerical': ['12_12'],
            'exit': []
        },
    }

    DETECTORS_209 = {
        0: {
            'boolean': ['63_1'],
            'saturation': ['63_4'],
            'numerical': ['63_7'],
            'exit': []
        },
        2: {
            'boolean': ['63_2'],
            'saturation': ['63_5'],
            'numerical': ['63_8'],
            'exit': []
        }
    }